@@ -891,7 +891,7 @@ module ActionMailer
         default_values = self.class.default.map do |key, value|
           [
             key,
-            value.is_a?(Proc) ? instance_exec(&value) : value
+            compute_default(value)
           ]
         end.to_h
 
@@ -900,6 +900,16 @@ module ActionMailer
         headers_with_defaults
       end
 
+      def compute_default(value)
+        return value unless value.is_a?(Proc)
+
+        if value.arity == 1
+          instance_exec(self, &value)
+        else
+          instance_exec(&value)
+        end
+      end
+
       def assign_headers_to_message(message, headers)
         assignable = headers.except(:parts_order, :content_type, :body, :template_name,
                                     :template_path, :delivery_method, :delivery_method_options)
